%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% diagnostic_read.m	Script to read in PSI_ or QGPV data file in specified format. 
%			Uncompress data file if available and needed.
%
% twnh Aug 01
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function array = diagnostic_read(filename)

global N_rad N_azim N_layer machineformat precision

% open data file
flag = 0 ;
fid = fopen(filename, 'r', machineformat) ;
if (fid == -1)
	temp_filename = [filename, '.gz'] ;
	[temp_fid, message] = fopen(temp_filename, 'r', machineformat) ;
	if (temp_fid == -1)
		fprintf(1,'     ERROR!  Can''t open file [%s] or [%s].\n', filename, temp_filename) ;
		fprintf(1,'     SYSTEM MESSAGE:  [%s]\n', message) ;
		return
	else
		copyfile(temp_filename, 'temp.gz') ;
		unix('gunzip temp.gz') ;
		fid = fopen('temp', 'r', machineformat) ;
		flag = 1 ;
	end
end

% read data file
fprintf(1, '     Reading file [%s] ...  ', filename) ;
tmp1 = fread(fid, 1, 'float32') ;		% spare 4-byte character at start of file (feature of Fortran-written unformatted files)
array = fread(fid, N_rad*N_azim*N_layer, precision)';
tmp2 = fread(fid, 1, 'float32') ;		% spare 4-byte character at end of file (feature of Fortran-written unformatted files)

% close data file
fclose(fid) ;

if (flag == 1)
	fprintf(1, '     unzip ...  ') ;
	delete('temp') ;
end

array = reshape(array, N_rad, N_azim, N_layer) ;
fprintf(1, 'OK.\n') ;

return
