!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! laplacian.f90	Laplacian in cylindrical polar co-ordinatess for dissipation terms.
!		Subroutine calculates:
!					out = laplacian (in)
!		= d^2/dr^2 (in) + (1/r) * d/dr (in) + (1/r^2) * d^2/dtheta^2 (in).
!
!		Called by SUBROUTINE dissipation.
! 
! twnh 24Aug01 & pdw 5Apr02
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

SUBROUTINE laplacian(in, out)

! load global variables
USE grid_params

IMPLICIT NONE

! local variables
REAL (KIND=wp), DIMENSION(N_rad, N_azim, N_layer), INTENT(IN)	:: in				! input field (always streamfunction)
REAL (KIND=wp), DIMENSION(N_rad, N_azim, N_layer), INTENT(OUT)	:: out				! \nabla^2 (in)
REAL (KIND=wp), DIMENSION(N_rad)				:: fac1, fac2			! temporary variable
REAL (KIND=wp)							:: fac3				! temporary variable
INTEGER								:: i, j, ip, im, jp, jm		! indices

WRITE(*,*) '  laplacian...'

! reset out array and set work variables
out = 0.0_wp
fac1 = 1.0_wp/(delta_theta*delta_theta*rad_dist*rad_dist)					! for (1/r^2) * d^2/dtheta^2 (in)
fac2 = 1.0_wp/(2.0_wp*rad_dist*delta_r)								! for (1/r) * d/dr (in)
fac3 = 1.0_wp/(delta_r*delta_r)									! for d^2/dr^2 (in)

! calculation of Laplacian
DO i = 1, N_rad											! loop over radial direction
	ip = i+1
	im = i-1
	DO j = 1, N_azim									! loop over azimuthal direction
		jp = j+1
		jm = j-1
		IF (j == 1) THEN								! azimuthal periodicity
			jm = N_azim
		ELSEIF (j == N_azim) THEN 
			jp = 1
		ENDIF

		out(i,j,:) = out(i,j,:) + fac1(i)*(in(i,jp,:) - 2.0_wp*in(i,j,:) + in(i,jm,:))		! (1/r^2) * d^2/dtheta^2 (in)
		IF (i == 1) THEN									! innermost boundary
			out(i,j,:) = out(i,j,:) + fac2(i)*2.0_wp*(in(ip,j,:)-in(i,j,:))			! (1/r) * d/dr (in)
		ELSEIF (i == N_rad) THEN								! outermost boundary
			out(i,j,:) = out(i,j,:) + fac2(i)*2.0_wp*(in(i,j,:)-in(im,j,:))			! (1/r) * d/dr (in)
		ELSE											! interior point
			out(i,j,:) = out(i,j,:) + fac3*(in(ip,j,:) - 2.0_wp*in(i,j,:) + in(im,j,:))	! d^2/dr^2 (in)
			out(i,j,:) = out(i,j,:) + fac2(i)*(in(ip,j,:) - in(im,j,:))			! (1/r) * d/dr (in)
		ENDIF

	END DO											! end azimthual loop
END DO												! end radial loop

END SUBROUTINE laplacian
