!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! step_q.f90	Leapfrog time stepping scheme with Robert 3-level time filter to damp
!		the computational mode.
!		Subroutine calculates:
!					q_next  = step_q (q_prev, tend),
!					q_this' = step_q (q_prev, q_this, q_next).
!
!		Called by PROGRAM qgam.
! 
! twnh 15Aug01 & pdw 26Mar02
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

SUBROUTINE step_q(q_prev, q_this, q_next, tend)

! load global variables
USE grid_params

IMPLICIT NONE

! declaration of local variables
REAL (KIND=wp), DIMENSION(N_rad, N_azim, N_layer), INTENT(IN)		:: q_prev, tend		! input PV field and PV tendency
REAL (KIND=wp), DIMENSION(N_rad, N_azim, N_layer), INTENT(INOUT)	:: q_this		! input/output PV field
REAL (KIND=wp), DIMENSION(N_rad, N_azim, N_layer), INTENT(OUT)		:: q_next		! output PV field

WRITE(*,*) ' step_q...'

! leapfrog step
q_next = q_prev + 2.0_wp*delta_t*tend

! Robert 3-level time filter
q_this = q_this + Robert_filter_parameter*0.5_wp*(q_next + q_prev - 2.0_wp*q_this)

END SUBROUTINE step_q
